/*
 * IRRC server
 * Copyright (c) 2003 Jan Wassenberg
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * Contact info:
 *   Jan.Wassenberg@stud.uni-karlsruhe.de
 *   http://www.stud.uni-karlsruhe.de/~urkt/
 */

#include <stdio.h>
#include <stdlib.h>

#include "sysdep/sysdep.h"
#include "posix.h"
#include "irrc.h"


#define MAX_CLIENTS 16

static int server;
static int clients[MAX_CLIENTS] = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 };


static void server_cleanup(void)
{
	close(server);

	for(int i = 0; i < MAX_CLIENTS; i++)
		if(clients[i] != -1)
			close(clients[i]);
}


void server_send(uint code, uint reps)
{
	/* LIRC-compatible packet */
	static char buf[100];
	sprintf(buf, "%016x %02x b a\n", code, reps);

	/* send it to all clients */
	for(int i = 0; i < MAX_CLIENTS; i++)
	{
		if(clients[i] == -1)
			continue;

		if(!send(clients[i], buf, sizeof(buf), 0))
			clients[i] = -1;
	}
}


/* accept incoming connections */
static void* server_thread(void*)
{
	for(;;)
	{
		/* look for an empty slot */
		int i;
		for(i = 0; i < MAX_CLIENTS; i++)
			if(clients[i] == -1)
				break;

		/* none found - wait & try again */
		if(i == MAX_CLIENTS)
		{
			sleep(1);
			continue;
		}

		clients[i] = (int)accept(server, 0, 0);
	}

/*	return 0; */
}


void server_init()
{
	/* bind server socket */
	static struct sockaddr_in addr;
	addr.sin_family = AF_INET;
	addr.sin_port = htons(8765);
	addr.sin_addr.s_addr = INADDR_ANY;
	server = (int)socket(AF_INET, SOCK_STREAM, 0);
	if(bind(server, (struct sockaddr*)&addr, sizeof(struct sockaddr_in)))
	{
		display_msg("IRRC problem", "The IRRC driver is already running.\n");
		exit(1);
	}
	listen(server, 5);

	atexit(server_cleanup);

	pthread_create(0, 0, server_thread, 0);
}
